unit Motion;

interface

uses
 SysUtils, Forms, Graphics, Types; 

const
 N=250;
 N0=3;
 NS:Array[1..N0] of Integer = (0,100,100);
 N02=2;
 q:Single=1;
 b:Single=0.05;
 dt:Single=0.01;

type
 TMotion=array[1..N0,0..N+1] of Single;
 TColors=array[1..N0] of TColor;

var
 iCanvas:TCanvas;
 X0,pX0:Single;
 nX0,npX0:Single;
 X,pX:TMotion;
 nX,npX:TMotion;
 Colors:TColors;
 t:Single;

procedure Draw(const Scale:Single);
procedure InitMotion;
procedure NextMotion;
procedure Clear;


implementation

function Cub(const X:Single):Single;
begin
 Cub:=X*Sqr(X);
end;

procedure Draw;
var
 i,k:Integer;
 grY,grY1:Integer;
begin
 for k:=1 to N0 do
 begin
  iCanvas.Pen.Color:=Colors[k];
  for i:=1 to N+1 do
  begin
   grY:=Round(300 - Scale*X[k,Abs(N*Abs(N02-k) - i)]) + (N02-k)*i;
   grY1:=Round(300 - Scale*X[k,Abs(N*Abs(N02-k) - i + 1)]) + (N02-k)*(i-1);
   iCanvas.MoveTo(N*Abs(N02-k) + i,grY);
   iCanvas.LineTo(N*Abs(N02-k) + i-1,grY1);
  end;
 end;
end;

procedure InitMotion;
var
 i,k:Integer;
begin
 for K:=1 to N0 do
  for i:=0 to N+1 do
  begin
   X[k,i]:=0;
   pX[k,i]:=0;
  end;
 X0:=0;
 pX0:=0;
 Colors[1]:=clRed;
 Colors[2]:=clFuchsia;
 Colors[3]:=clYellow;
end;


procedure NextMotion;
var
 i,k:Integer;
 Accel:Single;
begin
{Initial conditions for the central element}
 Accel:=0;
 for k:=1 to N0 do
  Accel:=Accel + q*X[k,N] + b*pX[k,N];
 Accel:=Accel - q*N0*X0 - b*N0*pX0;
 pX0:=pX0 + Accel*dt;
 X0:=X0 + pX0*dt + 0.5*Accel*dt*dt;
{Initial conditions on the margins of the system}
 for k:=1 to N0 do
 begin
  nX[k,NS[k]-1]:=0; npX[k,NS[k]-1]:=0;
  nX[k,N+1]:=X0; npX[k,N+1]:=pX0;
 end;
 if t<100 then
 begin
  nX[1,NS[1]]:=Sin(pi*t/10);
  //nX[2,0]:=-Sin(pi*t/10)
 end;
 {Differential equations solving itself}
 for k:=1 to N0 do
  for i:=1 to N do
  begin
   {The force acting on each particle}
   Accel:=q*(X[k,i-1] + X[k,i+1] - 2*X[k,i]) + b*(pX[k,i-1] + pX[k,i+1] - 2*pX[k,i]);
   {Eiler's method for differentila equations}
   npX[k,i]:=pX[k,i] + Accel*dt;
   nX[k,i]:=X[k,i] + pX[k,i]*dt + 0.5*Accel*dt*dt;
  {Other}
  end;
 {Non-linear effects and other}
 for k:=1 to N0 do
  for i:=0 to N+1 do
  begin
   X[k,i]:=nX[k,i];
   pX[k,i]:=npX[k,i];
  end;
end;

procedure Clear;
var
 iRect:TRect;
 TC:TColor;
begin
 TC:=iCanvas.Brush.Color;
 iCanvas.Brush.Color:=clBlack;
 iRect:=Rect(0,0,800,600);
 iCanvas.FillRect(iRect);
 iCanvas.Brush.Color:=TC;
end;

end.
